(***********************************************************************)
(*                                                                     *)
(*                           FaCiLe                                    *)
(*                 A Functional Constraint Library                     *)
(*                                                                     *)
(*            Nicolas Barnier, Pascal Brisset, LOG, CENA               *)
(*                                                                     *)
(* Copyright 2001 CENA. All rights reserved. This file is distributed  *)
(* under the terms of the GNU Lesser General Public License.           *)
(***********************************************************************)
(* $Id: fcl_binary.ml,v 1.9 2001/05/18 11:18:34 barnier Exp $ *)

open Fcl_var

class binary var1 var2 nogoods =
  let min = Fd.min var1
  and nogoods1 =
    Array.init (Fd.max var1 - Fd.min var1+1)
      (fun i ->
	(List.fold_right
	   (fun (x1, x2) acc -> if x1 = i+Fd.min var1 then x2::acc else acc) nogoods [])) in

 object
  inherit Fcl_cstr.t Fcl_cstr.normal
  method name = "binary"
  method delay x =
    delay [Attr.on_refine] var1 x;
    delay [Attr.on_refine] var2 x
  method update =
    if not (Fcl_stak.get solved) then begin
      match Fd.value var1 with
	Unk attr ->
	  Attr.iter
	    (fun x ->
	      try
	    	Fd.iter
		  (fun y -> if not (List.mem y nogoods1.(x-min)) then raise Exit) var2;
	    	match Fd.value var1 with
		  Unk var1_ -> Fd.refine var1  (Fcl_domain.remove x (Attr.dom var1_))
	    	| Val xx -> assert(x = xx); Fcl_stak.fail "binary"
	      with
	    	Exit -> ()) attr
      |	Val x ->
	  Fd.iter
	    (fun y ->
	      if List.mem y nogoods1.(x-min)
	      then match Fd.value var2 with
		Val yy -> assert(y = yy); Fcl_stak.fail "binary2"
	      |	Unk var2_ -> Fd.refine var2 (Fcl_domain.remove y (Attr.dom var2_)))
	    var2;
	  Fcl_stak.set solved true
    end
end

open Fcl_arith


let cstr var1 var2 ng =
  let c1 = Fcl_reify.boolean ~delay_on_negation:false (new binary var1 var2 ng)
  and c2 = Fcl_reify.boolean ~delay_on_negation:false (new binary var2 var1 (List.map (fun (x,y)-> (y,x)) ng)) in

  fd2e c1 +~ fd2e c2 =~ i2e 2
