(***********************************************************************)
(*                                                                     *)
(*                           FaCiLe                                    *)
(*                 A Functional Constraint Library                     *)
(*                                                                     *)
(*            Nicolas Barnier, Pascal Brisset, LOG, CENA               *)
(*                                                                     *)
(* Copyright 2001 CENA. All rights reserved. This file is distributed  *)
(* under the terms of the GNU Lesser General Public License.           *)
(***********************************************************************)
(* $Id: fcl_reify.mli,v 1.18 2001/06/19 13:15:09 barnier Exp $ *)

(* Module [Reify]: Constraints Reification *)

(*q All the reification functions and operators only work on
   REIFIABLE constraints, i.e. constraints which have a [check] function
   AND a [not] function (or built-in constraints for which the
   documentation does not mention "Not reifiable"). Otherwise a [Failure]
   ("fatal error") exception is raised. *)

val boolean : ?delay_on_negation:bool -> Fcl_cstr.t -> Fcl_var.Fd.t
 (* [boolean (?delay_on_negation:true) c] returns a boolean (0..1)
 variable associated to the constraint [c]. The constraint is verified
 iff the boolean is equal to 1. The waking conditions of the contraint
 relating [c] and the boolean are the ones set by the [delay] function
 of [c] (set by the [delay] argument of [Cstr.create]). If the optional
 argument [delay_on_negation] is set to [true], the new constraint is
 also attached to the events of the negation of [c] (i.e. the
 constraint returned by the [not] function of [c]), otherwise it is not.
 [delay_on_negation] default value is [true]. *)

val cstr : ?delay_on_negation:bool -> Fcl_cstr.t -> Fcl_var.Fd.t -> Fcl_cstr.t
 (* [cstr delay_on_negation c b] is equivalent to the
    constraint [boolean ?delay_on_negation c =~ b].
    [delay_on_negation] default value is [true]. *)

(*** Operators *)
val (&&~~) : Fcl_cstr.t -> Fcl_cstr.t -> Fcl_cstr.t
val (||~~) : Fcl_cstr.t -> Fcl_cstr.t -> Fcl_cstr.t
val (=>~~) : Fcl_cstr.t -> Fcl_cstr.t -> Fcl_cstr.t
val (<=>~~) : Fcl_cstr.t -> Fcl_cstr.t -> Fcl_cstr.t
val xor : Fcl_cstr.t -> Fcl_cstr.t -> Fcl_cstr.t
val not : Fcl_cstr.t -> Fcl_cstr.t
(* Logical reification operators on constraints, namely
   and, or, implies, equivalent, exclusive or, not. *)
