(***********************************************************************)
(*                                                                     *)
(*                           FaCiLe                                    *)
(*                 A Functional Constraint Library                     *)
(*                                                                     *)
(*            Nicolas Barnier, Pascal Brisset, LOG, CENA               *)
(*                                                                     *)
(* Copyright 2001 CENA. All rights reserved. This file is distributed  *)
(* under the terms of the GNU Lesser General Public License.           *)
(***********************************************************************)
(* $Id: fcl_stak.mli,v 1.14 2001/06/19 13:17:18 barnier Exp $ *)

(* Module [Stak]: Global Stack of Goals, Backtrackable Operations *)

(*q This module provides functions to control the execution of the goal
   stack, as well as backtrackable references. Nota: the module name
   [Stak] lacks a '[c]' because of a possible clash with the Ocaml
   standard library module [Stack]. *)

(*{*)
class virtual gl : object
  method virtual name : string
  method virtual call : gl option
end
(* Concrete type of goals. Hidden in Facile. *)
exception Empty_stack
(* _Undocumented_ *)
(*}*)
type level
(* Type of a level in the stack. *)
val older : level -> level -> bool
(* [older l1 l2] true if level [l1] precedes [l2]. *)

exception Level_not_found of level
(* Raised by [cut] and [cut_bottom]. *)

(*{*)
val reset : unit -> unit
(* Empty the stack. *)
(*}*)
val size : unit -> int
(* Size of the stack. *)
(*{*)
val save : gl list -> level
(* Push a choice point on the stack. *)
(*}*)
val level : unit -> level
(* Returns the current level. *)
(*{*)
val backtrack : unit -> gl list
  (* _Undocumented_
     Pop a success continuation. May raise Empty_stack. *)
val backtrack_all : unit -> unit
  (* _Undocumented_
     Pop the whole stack. *)
val trail : (unit -> unit) -> unit
  (* _Undocumented_
     [trail undo] Push the closure [undo] on the stack. The closure will
     be called when poped from the stack. *)
val cut_bottom : level -> unit
  (* _Undocumented_
     Raise Level_not_found if level is not reached and stack is empty. *)
(*}*)
val cut : level -> unit
  (* [cut l] cuts the choice points left on the stack until level [l].
     Raise [Level_not_found] if level is not reached and stack is empty. *)


type 'a ref
(* Backtrackable reference of type ['a]. *)
val ref : 'a -> 'a ref
(* Returns a reference which modification will be trailed during the
   solving of a goal. *)
val set : 'a ref -> 'a -> unit
(* Sets a reference. *)
val get : 'a ref -> 'a
(* Dereference. *)
val incr : int ref -> unit
val decr : int ref -> unit
(* Increments (resp. decrements) a reference. *)
(*{*)
val unsafe_set : 'a ref -> 'a -> unit
(* _Undocumented_
   Unbacktrackable modification. *)

val array_set : 'a array -> int -> 'a -> unit
(* _Undocumented_
   Bactrackable array assign. Used in [Fcl_ac6]. *)
(*}*)
(*** Control of Failure *)
exception Fail of string
(* Raised during solving whenever a failure occurs. The string argument
   is informative. *)
val fail : string -> 'a
(* [fail x] equivalent to [raise (Fail x)]. *)
